(provide "rc-mode")
(in-package "editor")


(export '(rc-mode
          *rc-tab-always-indent* *rc-indent-tabs-mode*
          rc-indent-level rc-comment-indent rc-continued-statement-offset rc-argdecl-indent
          rc-brace-offset rc-brace-imaginary-offset rc-label-offset
          ))

(defvar *rc-tab-always-indent* t)
(defvar *rc-indent-tabs-mode* t)
(unless (boundp 'rc-indent-level)
  (setq rc-indent-level 2)
  (setq rc-continued-statement-offset 2)
  (setq rc-argdecl-indent 5)
  (setq rc-brace-offset 0)
  (setq rc-brace-imaginary-offset 0)
  (setq rc-label-offset -2)
  (setq rc-comment-indent 2))

(defvar *rc-mode-hook* nil)
(defvar *rc-keyword-hash-table* nil)
(defvar *rc-keyword-file* "RigidChips")


(defvar *rc-mode-map* nil)
(unless *rc-mode-map*
  (setq *rc-mode-map* (make-sparse-keymap))
  (define-key *rc-mode-map* #\{ 'rc-electric-insert)
  (define-key *rc-mode-map* #\} 'rc-electric-close)
  (define-key *rc-mode-map* #\TAB 'rc-indent-line)
  (define-key *rc-mode-map* #\RET 'rc-newline-and-indent)
  )



(defvar *rc-mode-syntax-table* nil)
(unless *rc-mode-syntax-table*
  (setq *rc-mode-syntax-table* (make-syntax-table))
  (set-syntax-match *rc-mode-syntax-table* #\( #\))
  (set-syntax-match *rc-mode-syntax-table* #\{ #\})
  (set-syntax-string *rc-mode-syntax-table* #\")
  (set-syntax-symbol *rc-mode-syntax-table* #\_)
  (set-syntax-start-c++-comment *rc-mode-syntax-table* #\/)
  (set-syntax-end-c++-comment *rc-mode-syntax-table* #\LFD))



(defun rc-indent-line ()
  (interactive "*")
  (if (or (not (interactive-p))
          *rc-tab-always-indent*
          (save-excursion
            (skip-chars-backward " \t")
            (bolp)))
      (case (save-excursion
              (goto-bol)
              (parse-point-syntax))
        (:string)
        (:comment
         (let ((column (calc-c-comment-indent)))
           (when (integerp column)
             (smart-indentation column))))
        (t
         (let ((column (calc-rc-indent)))
           (when (integerp column)
             (smart-indentation column)))))
    (insert "\t"))
  t)



(defun calc-rc-indent ()
  (if (> (current-line-number) 1)
      (let* ((idt 0))
        (save-excursion
          (back-to-indentation)
          (if (char= (following-char) #\} )
              (setq idt (progn (goto-matched-parenthesis) (calc-indent-point)))
            (let ((tmp))
              (while(and(char/=(preceding-char) #\{ )(char/=(preceding-char) #\} )(char/=(preceding-char) #\NUL ))(backward-char))
              (if (char= (preceding-char) #\{ )
                  (setq idt (+ (calc-indent-point) rc-indent-level ))
                (if (char= (preceding-char) #\} )
                    (setq idt (calc-indent-point)))))))
        idt)
    0)) 

(defun calc-indent-point()
  (let* ((idt 0))
    (goto-bol)
    (while (or (char= (following-char) #\SPC) (char= (following-char) #\TAB) )
      (if (char= (following-char) #\SPC) (setq idt (1+ idt)))
      (if (char= (following-char) #\TAB) (setq idt (+ idt (tab-columns))))
      (forward-char))
    idt))



(defun rc-newline-and-indent (&optional (arg 1))
  (interactive "*p")
  (delete-trailing-spaces)
  (insert #\LFD arg)
  (rc-indent-line))


(defun rc-electric-insert (&optional (arg 1))
  (interactive "*p")
  (unless (prog1
              (parse-point-syntax)
            (self-insert-command arg))
    (rc-indent-line))
  t)

(defun rc-electric-close (&optional (arg 1))
  (interactive "*p")
  (unless (prog1
              (parse-point-syntax)
            (self-insert-command arg))
    (rc-indent-line))
  (save-excursion
    (forward-char -1)
    (and (goto-matched-parenthesis)
         (show-matched-parenthesis)))
  t)




(defun rc-mode ()
  (interactive)
  (kill-all-local-variables)
  (setq buffer-mode 'rc-mode)
  (setq mode-name "rc")
  (use-keymap *rc-mode-map*)
  (use-syntax-table *rc-mode-syntax-table*)
  (make-local-variable 'mode-specific-indent-command)
  (setq mode-specific-indent-command 'rc-indent-line)
  (make-local-variable 'indent-tabs-mode)
  (setq indent-tabs-mode *rc-indent-tabs-mode*)
  (make-local-variable 'c-comment-indent-variable)
  (setq c-comment-indent-variable 'rc-comment-indent)
  (and *rc-keyword-file*
       (null *rc-keyword-hash-table*)
       (setq *rc-keyword-hash-table*
             (load-keyword-file *rc-keyword-file* t)))
  (when *rc-keyword-hash-table*
    (make-local-variable 'keyword-hash-table)
    (setq keyword-hash-table *rc-keyword-hash-table*))
  (run-hooks '*rc-mode-hook*))

